#!/usr/bin/env python3
# -----------------------------------------------------------------------------
# Improved representation of clouds in the atmospheric component
# LMDZ6A of the IPSL Earth system model IPSL-CM6A
# J.-B. Madeleine et al. - 2020
# Journal of Advances in Modeling Earth Systems
# 
# Source Codes and supporting files
# Figure 7, three top rows
# -----------------------------------------------------------------------------

from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid
import netCDF4
from pylab import *
import numpy as np

# LMDZ 5A
ncfile="data/LmdzOrAPctrl_SE_1980_1999_1M_histmth_radTOA_144x90.nc"
ncgcm = netCDF4.Dataset(ncfile)
SWdnTOA = np.nanmean(ncgcm.variables['SWdnTOA'][:],axis=0)
SWdnTOAclr = np.nanmean(ncgcm.variables['SWdnTOAclr'][:],axis=0)
SWupTOA = np.nanmean(ncgcm.variables['SWupTOA'][:],axis=0)
SWupTOAclr = np.nanmean(ncgcm.variables['SWupTOAclr'][:],axis=0)
topl = np.nanmean(ncgcm.variables['topl'][:],axis=0)
topl0 = np.nanmean(ncgcm.variables['topl0'][:],axis=0)
CREsw5A = (SWdnTOA - SWupTOA) - (SWdnTOAclr - SWupTOAclr)
CRElw5A = topl0 - topl

# LMDZ 6A
ncfile="data/LmdzOr6Actrl_SE_1980_1999_1M_histmth_radTOA_144x90.nc"
ncgcm = netCDF4.Dataset(ncfile)
SWdnTOA = np.nanmean(ncgcm.variables['SWdnTOA'][:],axis=0)
SWdnTOAclr = np.nanmean(ncgcm.variables['SWdnTOAclr'][:],axis=0)
SWupTOA = np.nanmean(ncgcm.variables['SWupTOA'][:],axis=0)
SWupTOAclr = np.nanmean(ncgcm.variables['SWupTOAclr'][:],axis=0)
topl = np.nanmean(ncgcm.variables['topl'][:],axis=0)
topl0 = np.nanmean(ncgcm.variables['topl0'][:],axis=0)
CREsw6A = (SWdnTOA - SWupTOA) - (SWdnTOAclr - SWupTOAclr)
CRElw6A = topl0 - topl

# OBS
ncfile='data/rstcre_CERES_000001-000012_ac_144x90.nc'
ncobs = netCDF4.Dataset(ncfile)
CREswOBS = np.nanmean(ncobs.variables['rstcre'][:],axis=0)
ncfile='data/rltcre_CERES_000001-000012_ac_144x90.nc'
ncobs = netCDF4.Dataset(ncfile)
CRElwOBS = np.nanmean(ncobs.variables['rltcre'][:],axis=0)

allresults = np.dstack((-CREswOBS,CRElwOBS,-CREsw6A,CRElw6A,
                        -CREsw5A,CRElw5A))

# -----------------------------------------------------------------------------

# Grid
longi = ncgcm.variables['lon'][:]
lati = ncgcm.variables['lat'][:]

fig, axes = plt.subplots(nrows=3, ncols=2)

pltcounter = 0
for plotid in axes.flat:
    
    pltcounter = pltcounter + 1
    if pltcounter == 1:
        plotid.set_title(r"a) CERES (-1)$\times$CRF$^{SW}$")
    elif pltcounter == 2:
        plotid.set_title(r"b) CERES CRF$^{LW}$")
    elif pltcounter == 3:
        plotid.set_title(r"c) LMDZ6A (-1)$\times$CRF$^{SW}$")
    elif pltcounter == 4:
        plotid.set_title(r"d) LMDZ6A CRF$^{LW}$")
    elif pltcounter == 5:
        plotid.set_title(r"e) LMDZ5A (-1)$\times$CRF$^{SW}$")
    elif pltcounter == 6:
        plotid.set_title(r"f) LMDZ5A CRF$^{LW}$")
        
    m = Basemap(projection='cyl', lon_0=0., resolution='c',ax=plotid)
    cltplot = allresults[:,:,pltcounter-1]
    
    cltplot_shift,longi_shift = shiftgrid(180., cltplot, longi, start=False)
    Lon, Lat = np.meshgrid(longi_shift,lati)
    x, y = m(Lon,Lat)

    clevs = np.array(
        [-10,-5,0,5,10,20,30,40,50,60,70,80,90,100,110,120]
        )
    cs = m.contourf(x,y,cltplot_shift,clevs,
                    norm = mpl.colors.BoundaryNorm(boundaries=clevs,
                                                    ncolors=256),cmap='jet')   
    m.drawcoastlines()    
    m.drawmapboundary()
    m.drawparallels(np.arange(-90.,95.,30.),
                    labels=[1,0,0,0])
    m.drawmeridians(np.arange(-180.,180.,60.),
                    labels=[0,0,0,1])

fig.subplots_adjust(right=0.83)
cbar_ax = fig.add_axes([0.85, 0.15, 0.008, 0.7])
fig.colorbar(cs, cax=cbar_ax, ticks=clevs)
plt.show()

# -----------------------------------------------------------------------------